import json
import os


class TailAdapter(object):

    def __init__(self, reference):
        self.reference = reference
        with open('models/config.json', 'r') as f:
            self.tails = json.load(f)['tails']
        self.host = os.uname()[1]
        important_tail = lambda (k, t): t['machine'] in [self.host, '*']
        self.config = filter(important_tail, self.tails.items())[0][1]
        self.dimensions = self.config['dimensions']

    def transform(self, indexes):
        shape = self.reference.shape
        if indexes.__class__ == slice:
            indexes = [indexes]
        while len(indexes) < len(shape):
            indexes.append(indexes[-1])
        related = zip(shape, indexes)
        adapt = lambda (sh, idx): (slice(*self.dimensions[str(sh)])
                                   if str(sh) in self.dimensions else idx)
        return tuple(map(adapt, related))

    def __setitem__(self, indexes, matrix):
        indexes = self.transform(indexes)
        self.reference[indexes] = matrix
        return self.reference[indexes]

    def __getitem__(self, indexes):
        indexes = self.transform(indexes)
        return self.reference[indexes]

    @property
    def name(self):
        return self.reference.name

    @property
    def shape(self):
        print 'shape: %s' % str(self.reference.shape)
        return self.reference.shape

    @property
    def vtype(self):
        print 'vtype: %s' % self.reference.vtype
        return self.reference.vtype

    @property
    def least_significant_digit(self):
        print 'least_significant_digit: %s' % self.reference.least_significant_digit
        return self.reference.least_significant_digit


def tailored(reference):
    tail = reference
    if not isinstance(tail, TailAdapter):
        tail = TailAdapter(tail)
    return tail
